const { openBrowser, goto, textBox, into, write, click, toRightOf, below, link, press, image, waitFor, closeBrowser, screenshot } = require('taiko');
const assert = require('assert');

(async () => {
  try {
    const taikoTests = process.env.TAIKO_TESTS || 'all'
    const username = process.env.SSO_USERNAME
    const pw = process.env.SSO_USER_PW
    const domain = process.env.DOMAIN
    const adminpanelUrl = 'https://admin.' + domain
    const grafanaUrl = 'https://grafana.' + domain
    const globalTimeout = 60000

    // https://docs.taiko.dev/api/setconfig/
    // setConfig( { observeTime: 1000});
    setConfig( { observeTime: 0, navigationTimeout: globalTimeout });

    console.log('Executing these tests: ' + taikoTests)
    console.log('• Login to admin panel')
    await openBrowser()
    // await screencast.startScreencast('output.gif')

    // Wait for manual screencast recording
    // await waitFor(5000)

    await goto(adminpanelUrl)
    await click('Login')
    await click('Login with OAS')
    await write(username, into(textBox('Username')))
    await write(pw, into(textBox('Password')))
    await click('Remember me')
    await click('Sign in')

    // Nextcloud
    if (taikoTests.includes('nextcloud') || taikoTests === 'all') {
      console.log('• Nextcloud and Onlyoffice')
      await click(link(below('nextcloud')))
      await click('Log in with OpenAppStack')
      await click('Continue with ' + username)

      // Close potential nextcloud modal
      await press('Escape')

      // Open document and type some text
      await click(link(), above('Add notes'))
      await click('Document')
      await press('Enter')

      let italicButtonId = '#id-toolbar-btn-italic'
      await waitFor($(italicButtonId), globalTimeout)

      // Activate italic button
      let buttonStateBefore = await evaluate($(italicButtonId), (elem) => {return elem.getAttribute('class')})
      await assert.ok(!buttonStateBefore.includes('active'))
      await click($(italicButtonId))
      let buttonStateAfter = await evaluate($(italicButtonId), (elem) => {return elem.getAttribute('class')})
      await assert.ok(buttonStateAfter.includes('active'))

      await press(['H', 'i', ' ', 'f', 'r', 'o', 'm', ' ', 't', 'a', 'i', 'k', 'o', '!', 'Enter'])

      // Deactivate italic finially
      await click($(italicButtonId))


      await waitFor(5000)
      await closeTab()
    }

    // Rocketchat
    if (taikoTests.includes('rocketchat') || taikoTests === 'all') {
      // Navigate to rocketchat
      // Breaks because of 2fa challenge which is not recieved
      // by email
      // https://open.greenhost.net/openappstack/openappstack/-/issues/819
      // await click(link(below('rocketchat')));
      // await click('Login via OpenAppStack');
      // await click('Continue with ' + username)
    }

    // Wordpress
    if (taikoTests.includes('wordpress') || taikoTests === 'all') {
      console.log('• Wordpress')
      await goto(adminpanelUrl)
      await click(link(below('wordpress')))
      await click('Log in')
      await click('Login with OpenID Connect')
      await click('Continue with ' + username)
      await assert.ok(await link('Dashboard').exists())
    }

    // Grafana
    if (taikoTests.includes('grafana') || taikoTests === 'all') {
      console.log('• Grafana')
      await goto(adminpanelUrl)
      await click(link(below('grafana')))
      await click('Sign in with OpenAppStack')
      await click('Continue with ' + username)

      // Node exporter dashboard
      // Couldn't select "Manage dashboards" from the sidebar menu easily,
      // so we just go there
      await goto(grafanaUrl + '/dashboards')
      await click(link('Nodes'))
      await('CPU Usage')
      // await waitFor(2000)

      // Explore Loki log messages
      await goto(grafanaUrl + '/explore')
      await click(image(toRightOf('Explore')))
      await click('Loki')
      await click('Log browser')
      await click('app')
      await click('cert-manager')
      await click('grafana')
      await click('Show logs')
    }

    // Wait a bit before closing browser
    // await waitFor(20000)
  } catch (error) {
    await screenshot()
    // await screencast.stopScreencast()
    console.error(error)
    process.exitCode = 1
  } finally {
    // await screencast.stopScreencast()
    await closeBrowser()
  }
})()
